import { transform } from "@astrojs/compiler";
import { AstroErrorCodes } from "../errors.js";
import { prependForwardSlash } from "../path.js";
import { AggregateError, viteID } from "../util.js";
import { createStylePreprocessor } from "./style.js";
const configCache = /* @__PURE__ */ new WeakMap();
async function compile({
  config,
  filename,
  moduleId,
  source,
  transformStyle
}) {
  var _a;
  let cssDeps = /* @__PURE__ */ new Set();
  let cssTransformErrors = [];
  const transformResult = await transform(source, {
    pathname: `/@fs${prependForwardSlash(moduleId)}`,
    projectRoot: config.root.toString(),
    site: (_a = config.site) == null ? void 0 : _a.toString(),
    sourcefile: filename,
    sourcemap: "both",
    internalURL: `/@fs${prependForwardSlash(
      viteID(new URL("../../runtime/server/index.js", import.meta.url))
    )}`,
    experimentalStaticExtraction: true,
    preprocessStyle: createStylePreprocessor(transformStyle, cssDeps, cssTransformErrors)
  }).catch((err) => {
    err.code = err.code || AstroErrorCodes.UnknownCompilerError;
    throw err;
  }).then((result) => {
    switch (cssTransformErrors.length) {
      case 0:
        return result;
      case 1: {
        let error = cssTransformErrors[0];
        if (!error.code) {
          error.code = AstroErrorCodes.UnknownCompilerCSSError;
        }
        throw cssTransformErrors[0];
      }
      default: {
        const aggregateError = new AggregateError(cssTransformErrors);
        aggregateError.code = AstroErrorCodes.UnknownCompilerCSSError;
        throw aggregateError;
      }
    }
  });
  const compileResult = Object.create(transformResult, {
    cssDeps: {
      value: cssDeps
    },
    source: {
      value: source
    }
  });
  return compileResult;
}
function isCached(config, filename) {
  return configCache.has(config) && configCache.get(config).has(filename);
}
function getCachedSource(config, filename) {
  if (!isCached(config, filename))
    return null;
  let src = configCache.get(config).get(filename);
  if (!src)
    return null;
  return src.source;
}
function invalidateCompilation(config, filename) {
  if (configCache.has(config)) {
    const cache = configCache.get(config);
    cache.delete(filename);
  }
}
async function cachedCompilation(props) {
  const { config, filename } = props;
  let cache;
  if (!configCache.has(config)) {
    cache = /* @__PURE__ */ new Map();
    configCache.set(config, cache);
  } else {
    cache = configCache.get(config);
  }
  if (cache.has(filename)) {
    return cache.get(filename);
  }
  const compileResult = await compile(props);
  cache.set(filename, compileResult);
  return compileResult;
}
export {
  cachedCompilation,
  getCachedSource,
  invalidateCompilation,
  isCached
};
